#
# Copyright (C) 2002 Jp Calderone.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# System imports
import os, time

# Twisted imports
from twisted.python import reflect

# Pynfo imports
from Pynfo import error


def info_COMMANDS(bot, user, channel, args):
    """Usage: commands - List available commands.  -/+/* prefixes indicate disabled, privileged, and secure commands respectively."""
    cmds = reflect.prefixedMethodNames(bot.__class__, 'info_')
    pcmds = reflect.prefixedMethodNames(bot.__class__, 'privinfo_')
    scmds = reflect.prefixedMethodNames(bot.__class__, 'secure_')

    cmds.extend(bot.plugins['info_'].keys())
    pcmds.extend(bot.plugins['privinfo_'].keys())
    scmds.extend(bot.plugins['secure_'].keys())

    map(list.sort, (cmds, pcmds, scmds))
    cmds = [((x.lower() in bot.network.disabled and '-' or '') + x) for x in cmds] 
    pcmds = [((x.lower() in bot.network.disabled and '-+' or '+') + x) for x in pcmds]
    scmds = [((x.lower() in bot.network.disabled and '-*' or '*') + x) for x in scmds]
    bot.reply(channel, "Commands: %s" % (', '.join(cmds + pcmds + scmds).lower(),))


def info_PLUGINS(bot, user, channel, args):
    """Usage: plugins - List all loaded plugin modules."""
    f = lambda f: os.path.splitext(os.path.basename(f))[0]
    bot.reply(
        channel, 
        "Plugins: %s" % (
            ', '.join(
                [f(m.__file__) for m in bot.factory.pluginModules]
            ),
        )
    )

def info_SOURCE(bot, user, channel, args):
    """Usage: source - Reveal the location of my source code"""
    bot.reply(channel, "My source is available at " + bot.SRC_URL)


def info_HELP(bot, user, channel, args):
    """Usage: help <command> - Give help for <command>"""
    if len(args.strip()) == 0:
        raise error.UsageError("Get help on what?")

    args = args.upper()
    cmd = getattr(
        bot, 'info_' + args,
        getattr(
            bot, 'privinfo_' + args,
            getattr(bot, 'secure_' + args,
                bot.plugins['info_'].get(args,
                    bot.plugins['privinfo_'].get(args,
                        bot.plugins['secure_'].get(args, None)
                    )
                )
            )
        )
    )
    if cmd is None:
        raise error.UsageError("There is no such command.")
    elif cmd.__doc__ is None:
        raise error.UsageError("There is no help for that command.")
    else:
        bot.reply(channel, cmd.__doc__)


def info_CHANNELS(bot, user, channel, args):
    """Usage: channels - Reports the channels I am currently in"""
    bot = bot.factory.allBots.get(args, bot)
    bot.reply(channel, 'I am in: %s' % ', '.join(bot.network.channels))


def info_STATUS(bot, user, channel, args):
    """Usage: status - Display status information"""
    r = 'I was created at %s UTC and have been connected to %s since %s UTC.  Since then I have been rebuilt %d tim%s and served %d quer%s.'
    r = r % (
        time.asctime(time.gmtime(bot.factory.createTime)),
        bot.network.alias, time.asctime(time.gmtime(bot.starttime)),
        bot.rebuilds, (bot.rebuilds == 1 and 'e' or 'es'), bot.queries,
        (bot.queries == 1 and 'y' or 'ies')
    )
    f = len(bot.factory.allBots)
    r = r + '  I am connected to %d IRC networ%s: %s.'
    r = r % (
        f,
        f == 1 and 'k' or 'ks',
        ', '.join(bot.factory.data['networks'].keys())
    )

    f = bot.factory.countFactoids()
    r = r + '  There %s %d factoi%s in my database.' % (f == 1 and 'is' or 'are', f, f == 1 and 'd' or 'ds')

    r = r + '  I have transmitted %d bytes and received %d bytes.' % (bot.inBytes, bot.outBytes)
    r = r + '  Since my factory was created, a total of %d bytes have been transmitted and %d bytes have been received.' % (bot.factory.inBytes, bot.factory.outBytes)

    r = r + '  My average ping to this network is %0.2f seconds.' % bot.pingAve

    bot.reply(channel, r)
