#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

# System imports
import time

# Pynfo imports
from Pynfo import error, interfaces, info

# Twisted imports
from twisted.protocols.dns import str2time
from twisted.python import log

bans = None
calls = None
def init(factory):
    store = factory.getComponent(interfaces.IStorage)
    
    global bans
    try:
        bans = store.retrieve('oper', None)
    except error.RetrievalError:
        bans = {}

    global calls
    calls = []
    now = time.time()
    from twisted.internet import reactor
    for ban, info in bans.iteritems():
        calls.append(reactor.callLater(
            min(0, info[1] - (now - info[0])),
            clearBan, factory, ban
        ))


def fini(factory):
    store = factory.getComponent(interfaces.IStorage)
    store.store('oper', None, bans)
    for call in calls:
        # XXX - This is a memory leak, eh?
        try:
            call.cancel()
        except:
            pass


def clearBan(factory, (net, chan, mask)):
    try:
        bot = factory.allBots[net]
    except KeyError:
        from twisted.internet import reactor
        reactor.callLater(600, clearBan, factory, (net, chan, mask))
        log.msg("Not connected to %s - cannot clear ban" % net)
    else:
        bot.mode(chan, 0, 'b', mask=mask)
        del bans[(net, chan, mask)]


def privinfo_BAN(bot, user, channel, args):
    """Usage: ban [channel] <hostmask> <duration> <reason> - Set a ban with the given hostmask and record the given reason.  Remove it after the given duration."""
    net = bot.network.alias
    if user == channel:
        try:
            targchan, hostmask, duration, reason = args.split(None, 3)
        except ValueError:
            raise error.UsageError
    else:
        targchan = channel.lower()
        try:
            hostmask, duration, reason = args.split(None, 2)
        except ValueError:
            raise error.UsageError

    try:
        duration = str2time(duration)
    except ValueError, e:
        raise error.UsageError(str(e))

    hostmask = hostmask.lower()
    if bans.has_key((targchan, hostmask)):
        bot.reply(channel, "The hostmask is already banned: " + bans[(net, targchan, hostmask)][3])
    else:
        bans[(net, targchan, hostmask)] = time.time(), duration, reason, user

    bot.mode(targchan, 1, 'b', mask=hostmask)
    from twisted.internet import reactor
    calls.append(
        reactor.callLater(
            duration, clearBan, bot.factory, (net, targchan, hostmask)
        )
    )


def privinfo_BANINFO(bot, user, channel, args):
    """Usage: baninfo [channel] <hostmask> - Show the info about the ban on the given hostmask"""
    net = bot.network.alias
    if user == channel:
        try:
            targchan, hostmask = args.lower().split(None, 1)
        except TypeError:
            raise error.UsageError
    else:
        targchan = channel.lower()
        hostmask = args.lower()
        
    if bans.has_key((net, targchan, hostmask)):
        b = bans[(net, targchan, hostmask)]
        fmt = "%r was banned at %s (%s remaining) by %s: %s"
        left = b[1] - (time.time() - b[0])
        args = (hostmask, time.ctime(b[0]), info.formatTime(left), b[3], b[2])
        bot.reply(channel, fmt % args)
    else:
        bot.reply(channel, "%s is not banned." % args)


def privinfo_BANLIST(bot, user, channel, args):
    """Usage: banlist [channel] - List the bans for the given channel"""
    if user == channel or args:
        targchan = args.lower()
    else:
        targchan = channel.lower()

    if not targchan:
        raise error.UsageError

    reported = 0
    for (net, chan, hostmask), ban in bans.iteritems():
        if chan == targchan and net == bot.network.alias:
            bot.reply(channel, "%r by %s" % (hostmask, ban[3]))
            reported = 1
    if not reported:
        bot.reply(channel, "No bans on that channel")


def privinfo_OP(bot, user, channel, query):
    """Usage: op [ user [ @network ] ] - Mode +o the specified user"""
    if not query:
        bot.mode(channel, 1, 'o', user=user)
    else:
        parts = query.split('@', 1)
        if len(parts) == 1:
            bot.mode(channel, 1, 'o', user=parts[0])
        elif len(parts) == 2:
            target, network = parts
            if not bot.factory.allBots.has_key(network):
                bot.reply(channel, "No such network.")
            else:
                bot.factory.allBots[network].mode(channel, 1, 'o', user=target)
        else:
            bot.reply(channel, "Usage error.  See 'help op'")

def privinfo_KICK(bot, user, channel, query):
    """Usage: kick < user [ @network ] > [ reason ] - Kick a user from the channel"""
    if not query:
        raise error.UsageError
    parts = query.split(None, 1)
    reason = None
    if len(parts) == 2:
        reason = parts[1]
    parts = parts[0].split('@', 1)
    if len(parts) == 2:
        try:
            bot = bot.factory.allBots[parts[1]]
        except KeyError:
            raise error.UsageError, "No such network"

    bot.kick(channel, parts[0], reason)
    bot.reply(user, 'Okay')
