#
#   rfcindex - An rfc searching plugin for Twisted Pynfo
#   Copyright (C) 2003-01-24 Thomas Raschbacher (lordvan@lordvan.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

from twisted.web import client

from Pynfo import error, interfaces

import types, os

INDEX_URL = "ftp://ftp.isi.edu/in-notes/rfc-index.txt"
TEMPLATE_URL = "http://www.rfc-editor.org/rfc/rfc%d.txt"

class RFCIndex:
    """I am a class to search the rfc-index."""
    rfcs = None
    def __init__(self, filename=None):
        self.rfcs = {}
        if filename and os.path.exists(filename):
            self._rfcsep(file(filename).readlines())
        else:
            client.getPage(INDEX_URL).addCallback(self._rfcsep)


    def _rfcsep(self, lines = None):
        if lines: self.lines = lines
        rfc=""
        rfcnum=-1
        import re
        pat=re.compile('^(\d{4}) ')
        inRFC=0
        for l in self.lines:
            m = pat.match(l)
            if m:
                inRFC = 1
                rfc = l
                rfcnum = m.groups()[0]           
            elif inRFC == 1:
                if len(l.strip()) > 0:
                    rfc += l
                else:
                    inRFC = 0
                    self.rfcs[rfcnum] = rfc

    def __len__(self):
        return len(self.rfcs)

    def has_key(self, key):
        if isinstance(key, str):
            # it's a string so pretend the one how does this knows what he does
            if len(key) == 4:
                return self.rfcs.has_key(key)
            else:
                raise KeyError, "String has to be length 4"
        elif isinstance(key, int):
            return self.rfcs.has_key('%04d' % key)
        else:
            raise KeyError, "Illegal type(only int or string with length 4)"
        return self.rfcs.has_key(key)

    def keys(self):
        return self.rfcs.keys()

    def values(self):
        return self.rfcs.values()

    def items(self):
        return self.rfcs.items()

    def __getitem__(self, hash):
        if isinstance(hash, str):
            # it's a string so pretend the one how does this knows what he does
            if len(hash) == 4:
                return self.rfcs[hash]
            else:
                raise KeyError, "String has to be length 4"
        elif isinstance(hash, int):
            return self.rfcs['%04d' % (hash)]
        else:
            raise KeyError, "Illegal type(only int or string with length 4)"

    def get(self, key, alt):
        if self.has_key(key):
            return self[key]
        else:
            return alt

    def searchNum(self, num):
        return self[num]
    
    def searchTerm(self, term, ignoreCase=0):
        matches=[]
        if ignoreCase:
            for i in self.rfcs:
                if self.rfcs[i].find(term) > -1:
                    matches.append(i)
        else:
            lterm = term.lower()
            for i in self.rfcs:
                if ((self.rfcs[i]).lower()).find(lterm) > -1:
                    matches.append(i)
        return matches

    
rfc = RFCIndex()
def info_RFC(bot, user, channel, query):
    """Usage: rfc <number or term> - lookup RFC by number or search term (ftp://ftp.isi.edu/in-notes/rfc-index.txt)"""
    if not query:
        raise error.UsageError
    try:
        query = int(query.strip())
        # check if we got a number
        bot.reply(
            channel,
            "%s %s" % (
                rfc.get(query, "Not Found"),
                TEMPLATE_URL % query
            )
        )
    except ValueError:
        rfcterm = rfc.searchTerm(query)
        if not rfcterm:
            bot.reply(channel, "Not Found")
        else:
            rfcterm.sort()
            ret = ', '.join(rfcterm[:39])
            ret = '%s matches in RFCs: %s' % (len(rfcterm), ret)
            bot.reply(channel, ret)

#def init(factory):
#    sched = factory.getComponent(interfaces.IScheduler)
#    sched.schedule(interfaces.IScheduler.WEEKLY, fetchLatest)
#
#    if not os.path.exists('rfc.index'):
#        pass

# test case if we get executed
if __name__ == '__main__':
    rfc = RFCIndex('/var/cache/rfc/rfc-index')
    print rfc[2]
    print rfc.searchNum(4)
    print rfc['0231']
    print rfc['9999']
    try:
        print rfc['sa']
    except KeyError, e:
        print "Key Error:", e
        pass
    try:
        print rfc[{}]
    except KeyError, e:
        print "Key Error:", e
        pass
    m1 = rfc.searchTerm('dhcp', 1)
    print len(m1), m1
    for m in m1:
        print rfc[m]

    m2 = rfc.searchTerm('dhcp')
    print len(m2), m2
    for m in m2:
        print rfc[m]
