#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

from twisted.web import widgets, resource, static

class Redirect(resource.Resource):
    def __init__(self, destination):
        self.destination = destination


    def render(self, request):
        return static.redirectTo(self.destination, request)


class RedirectionList(resource.Resource):
    def __init__(self, redirects):
        l = []
        for url in redirects.values():
            l.append('<a href="%s">%s</a><br>' % (url, url))
        self.output = '\n'.join(l)


    def render(self, request):
        return self.output


class Shortener(resource.Resource):
    def __init__(self, factory):
        resource.Resource.__init__(self)
        self.mapping = factory.shortenMap


    def __getattr__(self, name):
        try:
            return self.__dict__[name]
        except:
            raise AttributeError, name


    def listDynamicNames(self):
        return self.mapping.keys()


    def listDynamicEntities(self):
        return self.mapping.items()


    def getChild(self, path, request):
        if path == '':
            return RedirectionList(self.mapping)
        elif self.mapping.has_key(path):
            return Redirect(self.mapping[path])
        return resource.Resource.getChild(self, path, request)
